#include "auth/securitymessage.h"
#include <alarm.h>
#include <mach/mc13224v/aes_controller.h>

const unsigned int DATA_SIZE = 32;
const unsigned int MAC_SIZE = 16;

SecurityMessage::SecurityMessage(){
    df = new DiffieHellman;
    c = new Cryptography();
}

DiffieHellman* SecurityMessage::getDiffieHellman(){
    return df;
}

/*long long int SecurityMessage::generateKey()
{
    long long int y = df->obterY();
    //mandar Y pro server (se for sensor) e pro sensor (se for server)
    //receber yb do server (se for sensor) e do sensor (se for server)
    long long int yb;
    df->calcularChave(yb);
    return df->getKey(); //chave usada pra cifrar/decifrar a temperatura
}*/

/**
    Cifra e envia a msg
 */
bool SecurityMessage::send(char* msg, TCP::Channel* client, char * address, int port)
{

    const unsigned int DATA_SIZE = 32;
    const unsigned int MAC_SIZE = 16;

    char mac_msg[MAC_SIZE+DATA_SIZE];
    char keyChar[10];
    char encrypted[DATA_SIZE];

    long long int key = df->getKey();
    itoa(key, keyChar);
    c->encrypt(msg, keyChar, encrypted, mac_msg);

    scout << "Msg cifrada " << encrypted << endl;

    strcat(mac_msg, encrypted);
    scout << "Msg concatenda com a assinatura: " << mac_msg << endl;

    if(client->connect(TCP::Address(address, port))){
        Alarm::delay(1000000);
        scout << "Enviando " << endl;
        client->send(mac_msg, sizeof(mac_msg));
        scout << "Msg enviada " << endl;
        scout << "Fechando conexao";
        Alarm::delay(1000000);
        client->close();

    } else
        scout << "Falha no envio da msg" << endl;
}

/**
    Recebe a msg e a decifra
 */
char* SecurityMessage::receive()
{
    TCP::Channel socket;
    socket.bind(1337);

    char msgReceived[DATA_SIZE];
    char chave[10];
    long long int key = df->getKey();
    itoa(key, chave);

    while(1){
        scout << "Escutando " << endl;
        while(!socket.listen());
        scout << "Recebendo msg" << endl;
        socket.receive(msgReceived, sizeof(msgReceived));

        scout << "Msg received : " << msgReceived << endl;

        c->decrypt(msgReceived, "123", decrypted);

        scout << "Msg decriptada: " << decrypted << endl;
        socket.close();
    }
    return decrypted;

}

void SecurityMessage::keyHandShakeServer(TCP::Channel * socket)
{
    char yReceived[8]; //char buffer[256];
    char reply[8] = {};

    scout << "Escutando " << endl;
    while (!socket->listen());
    scout << "Escutei algo" << endl;

    Alarm::delay(500000);
    socket->receive(yReceived, sizeof(yReceived));
    Alarm::delay(3000000);
    long yb = atol(yReceived);
    scout << "Y received: " << yb << endl;
    long long int myY = df->obterY();
    df->calcularChave(yb);
    scout << "Chave gerada: " << df->getKey() << endl;

    itoa(myY, reply);
    int reply_size = sizeof(reply);

    scout << "Mandando minha chave" << endl;
    socket->send(reply, reply_size);
    Alarm::delay(100000);
    socket->close();
    scout << "Fechou conexao" << endl;
}

void SecurityMessage::keyHandShakeSensor(TCP::Channel * client){

    long long int y = df->obterY();
    char ybStr[8];
    scout << "Inicializando HandShake" << endl;
    Alarm::delay(5000000);
    bool conectou =false;
    while(!conectou){
        scout << "Tentando conexao" << endl;
        if(client->connect(TCP::Address("10.0.0.110", 1337))){
            char yStr[8];
            itoa(y, yStr);
            Alarm::delay(3000000);
            client->send(yStr, sizeof(yStr));
            scout << "Enviou yb" << endl;

            Alarm::delay(500000);
            client->receive(ybStr, sizeof(ybStr));
            long yb = atol(ybStr);
            scout << "Recebey yb " << yb << endl;
            df->calcularChave(yb);
            scout << "Chave gerada: " << df->getKey() << endl;
            conectou = true;
        }
    }

    Alarm::delay(1000000);
    client->close();

    scout << "Sensor finalizou HandShake" << endl;
}

long long int SecurityMessage::getKey() {
	return df->getKey();
}
